#ifndef LITTLE_GC_H
#define LITTLE_GC_H

#ifdef __cplusplus
extern "C" {
#endif

/*
 * Initializes GC system.
 */
void ltl_gc_init(
	ltl_threadenv_t *	/* Thread environment */
	);
/*
 * Disposes the GC internal resources.
 */
void ltl_gc_dispose();
/*
 * Allocates a block of GC watched ALICE objects.
 */
ltl_object_t *
	ltl_gc_alloc(
		ltl_sym_t,	/* Owning type ID - which object owns the fields? */
		ltl_word_t	/* Number of entries in the block */
	);
/*
 * Reallocates a previously allocated, GC watched block.
 */
ltl_object_t *
	ltl_gc_realloc(
		ltl_object_t *, /* The block to be reallocated (grown) */
		ltl_word_t)	/* The new amount of entries */
	;
/*
 * This call locks a location in memory, ie. it won't be garbage collected
 * until a call of ltl_gc_unlock() is issued on the same location.
 */
ltl_object_t *
	ltl_gc_lock(
		ltl_object_t *	/* The location to protect. */
	);
/*
 * This call makes a (previously locked) location collectable again.
 */
ltl_object_t *
	ltl_gc_unlock(
		ltl_object_t *	/* The location to unprotect. */
	);

/*
 * Enters a critical section, which must not be interrupted
 * by GC runs.
 */
void
	ltl_gc_barrier_enter();

/*
 * Leaves a critical section.
 */
void
	ltl_gc_barrier_exit();

/*
 * Starts a GC run.
 */
void ltl_gc_run();
/*
 * Starts a GC mark run.
 */
void ltl_gc_mark();
/*
 * Starts a finalization run.
 */
void ltl_gc_finalize();
/*
 * Starts a cleanup run.
 */
void ltl_gc_sweep();
/*
 * Returns the current amount of heap memory in use.
 */
size_t ltl_gc_getusage();
/*
 * Returns true, if the GC system is in verbose mode.
 */
int ltl_gc_isverbose();
/*
 * Activates/deactivates the GC verbose mode.
 */
void ltl_gc_setverbose(
	int	/* true, if GC should be verbose, else false */
	);
/*
 *
 */
int ltl_gc_getthreshold();
/*
 *
 */
void ltl_gc_setthreshold(
	int	/* new maximum bytes in use, before GC runs should be scheduled */
	);

#ifdef __cplusplus
}
#endif

#endif
